﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Genuilder.Extensibility.Tests
{
	[TestClass]
	public class CodeRepositoryTest
	{
		[TestMethod]
		public void ShouldTrackCodeItem()
		{
			MockCodeRepository repository = new MockCodeRepository();
			bool trackItem = false;
			repository.CodeItemCreated += (c) => trackItem = true;
			repository.Histories.Add(new CodeItemHistory(new MockCodeItem("", "file", CodeItemState.Exist)));
			repository.Initialize();
			Assert.IsTrue(trackItem);
		}

		[TestMethod]
		public void ShouldFireModifiedCodeItem()
		{
			MockCodeRepository repository = new MockCodeRepository();
			var item = new MockCodeItem("", "file", CodeItemState.Exist);
			bool itemModified = false;
			item.ContentChanged += (c) => itemModified = true;
			repository.Histories.Add(new CodeItemHistory(item)
			{
				Modified = true
			});
			repository.Initialize();
			Assert.IsTrue(itemModified);
		}

		[TestMethod]
		public void ShouldFireNewStateCodeItem()
		{
			MockCodeRepository repository = new MockCodeRepository();
			var item = new MockCodeItem("", "file", CodeItemState.Exist);
			bool itemStateChanged = false;
			item.StateChanged += (c, n, o) => itemStateChanged = true;
			repository.Histories.Add(new CodeItemHistory(item)
			{
				NewState = CodeItemState.Deleted
			});
			repository.Initialize();
			Assert.IsTrue(itemStateChanged);
		}
	}

}
