﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ICSharpCode.NRefactory.Ast;
using ICSharpCode.NRefactory.Parser;
using ICSharpCode.NRefactory;
using System.IO;
using ICSharpCode.NRefactory.PrettyPrinter;

namespace Genuilder.Extensibility.NRefactory
{
	public class CompilationUnitExtension
	{
		readonly CodeItem _Item;
		public CompilationUnitExtension(CodeItem item)
		{
			_MustParse = true;
			_Item = item;
			item.ContentChanged += new ContentChangedHandler(item_ContentChanged);
		}

		bool _MustParse;
		void item_ContentChanged(CodeItem sender)
		{
			_MustParse = true;
		}

		CompilationUnit _Unit;
		Errors _Errors;
		public CompilationUnit CompilationUnit
		{
			get
			{
				EnsureParsed();
				return _Unit;
			}
		}

		public bool IsSupported
		{
			get
			{
				if(_Item.Name.EndsWith(".cs", StringComparison.InvariantCultureIgnoreCase))
					return true;
				if(_Item.Name.EndsWith(".vb", StringComparison.InvariantCultureIgnoreCase))
					return true;
				return false;
			}
		}

		public Errors Errors
		{
			get
			{
				EnsureParsed();
				return _Errors;
			}
		}

		private void EnsureParsed()
		{
			if(_MustParse)
			{
				_MustParse = false;
				Parse();
			}
		}


		public void Save()
		{
			var visitor = GetOutputVisitor(_Item.Name);
			CompilationUnit.AcceptVisitor(visitor, null);
			_Item.Content = visitor.Text;
		}

		private IOutputAstVisitor GetOutputVisitor(string file)
		{
			if(file.EndsWith(".cs"))
				return new CSharpOutputVisitor();
			else if(file.EndsWith(".vb"))
				return new VBNetOutputVisitor();
			else
				return null;
		}


		private void Parse()
		{
			var parser = ParserFactory.CreateParser(GetLanguage(), new StringReader(_Item.Content));
			parser.Parse();
			_Unit = parser.CompilationUnit;
			_Errors = parser.Errors;
		}

		private SupportedLanguage GetLanguage()
		{
			if(_Item.Name.EndsWith(".cs", StringComparison.InvariantCultureIgnoreCase))
				return SupportedLanguage.CSharp;
			if(_Item.Name.EndsWith(".vb", StringComparison.InvariantCultureIgnoreCase))
				return SupportedLanguage.VBNet;
			throw new NotSupportedException("cannot parse file " + _Item.Name + ", it seems it's not VB nor CSharp");
		}
	}
}
