﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Build.Utilities;
using Microsoft.Build.Framework;
using System.IO;
using System.Reflection;

namespace Genuilder.Extensibility.MSBuild
{
	public class ExtensibilityTask : AppDomainIsolatedTask
	{
		public ITaskItem[] CompileIn
		{
			get;
			set;
		}
		public String OutputPath
		{
			get;
			set;
		}

		public ITaskItem[] Extensions
		{
			get;
			set;
		}

		[Output]
		public ITaskItem[] CompileOut
		{
			get;
			set;
		}

		public override bool Execute()
		{
			if(Extensions == null)
				return true;
			if(CompileIn == null)
				CompileIn = new TaskItem[0];
			var itemsByName = CompileIn.ToDictionary(d => d.ItemSpec);
			var historyPath = Path.Combine(OutputPath, "RunHistory.xml");
			GenEngine engine = new GenEngine();
			engine.Logger = new MSBuildLogger(this.Log);
			engine.StartInfo.GenerationDirectory = OutputPath;
			engine.SourceFiles.AddRange(CompileIn.Select(s => s.ItemSpec));
			engine.Extensions.AddRange(GetExtensions());
			engine.StartInfo.RunHistory = RunHistory.LoadOrNull(historyPath);
			var result = engine.Run(true);
			if(Log.HasLoggedErrors)
				return false;
			result.History.Save(historyPath);
			CompileOut = result.ExecutionContext.SourceFiles.Where(s => !itemsByName.ContainsKey(s)).Select(g => CreateTaskItem(g)).ToArray();
			return true;
		}

		private TaskItem CreateTaskItem(string sourceFile)
		{
			var item = new TaskItem(sourceFile);
			item.SetMetadata("Visible", "false");
			return item;
		}
		private List<IExtension> GetExtensions()
		{
			Dictionary<string, Assembly> loadedAssemblies = new Dictionary<string, Assembly>();
			return Extensions.Select(e => e.ItemSpec.ToExtension(loadedAssemblies)).ToList();
		}
	}
}
