﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Build.Utilities;

namespace Genuilder.Extensibility.MSBuild
{
	public class MSBuildLogger : IGenLogger
	{
		class MsBuildExtensionLogger : IExtensionLogger
		{
			private IExtension extension;
			private TaskLoggingHelper log;

			public MsBuildExtensionLogger(IExtension extension, TaskLoggingHelper log)
			{
				this.extension = extension;
				this.log = log;
			}
			#region IExtensionLogger Members

			public void Info(string message)
			{
				log.LogMessage(extension.ToString() + " : " + message);
			}

			public void Warn(string message)
			{
				log.LogWarning(extension.ToString() + " : " + message);
			}

			public void Error(string message)
			{
				log.LogError(extension.ToString() + " : " + message);
			}

			#endregion
		}
		public class MsBuildGenItemLogger : IGenItemLogger
		{
			private GenItem genItem;
			private TaskLoggingHelper log;

			public MsBuildGenItemLogger(GenItem genItem, TaskLoggingHelper log)
			{
				this.genItem = genItem;
				this.log = log;
			}

			#region IGenItemLogger Members

			public void Info(string message)
			{
				log.LogMessage(message);
			}

			public void Warn(string message)
			{
				Warn(message, 0, 0);
			}
			public void Warn(string message, int line, int column)
			{
				log.LogWarning(null, null, null, genItem.Name, line, column, line, column, message);
			}

			public void Error(string message)
			{
				Error(message, 0, 0);
			}
			public void Error(string message, int line, int column)
			{
				log.LogError(null, null, null, genItem.Name, line, column, line, column, message);
			}

			#endregion
		}
		private TaskLoggingHelper log;

		public MSBuildLogger(TaskLoggingHelper log)
		{
			this.log = log;
		}

		#region IGenLogger Members

		public IExtensionLogger CreateExtensionLogger(IExtension extension)
		{
			return new MsBuildExtensionLogger(extension, log);
		}


		public IGenItemLogger CreateGenItemLogger(GenItem genItem)
		{
			return new MsBuildGenItemLogger(genItem, log);
		}

		#endregion
	}
}
