﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Build.Utilities;
using Microsoft.Build.Framework;
using System.IO;
using System.Reflection;

namespace Genuilder.Extensibility.MSBuild
{
	public class ExtensibilityTask : AppDomainIsolatedTask
	{
		public ITaskItem[] CompileIn
		{
			get;
			set;
		}
		public ITaskItem[] NoneIn
		{
			get;
			set;
		}
		public ITaskItem[] PageIn
		{
			get;
			set;
		}
		public String OutputPath
		{
			get;
			set;
		}

		public ITaskItem[] Extensions
		{
			get;
			set;
		}

		[Output]
		public ITaskItem[] PageOut
		{
			get;
			set;
		}
		[Output]
		public ITaskItem[] CompileOut
		{
			get;
			set;
		}
		[Output]
		public ITaskItem[] NoneOut
		{
			get;
			set;
		}

		public override bool Execute()
		{
			if(Extensions == null)
				return true;
			if(CompileIn == null)
				CompileIn = new TaskItem[0];
			if(NoneIn == null)
				NoneIn = new TaskItem[0];
			if(PageIn == null)
				PageIn = new TaskItem[0];
			var itemsByName = CompileIn.ToDictionary(d => d.ItemSpec);
			var historyPath = Path.Combine(OutputPath, "RunHistory.xml");
			GenEngine engine = new GenEngine();
			engine.Logger = new MSBuildLogger(this.Log);
			engine.StartInfo.GenerationDirectory = OutputPath;
			engine.SourceFiles.AddRange(CompileIn.Select(s => new SourceFile(s.ItemSpec, SourceType.Compile)));
			engine.SourceFiles.AddRange(NoneIn.Select(s => new SourceFile(s.ItemSpec, SourceType.None)));
			engine.SourceFiles.AddRange(PageIn.Select(s => new SourceFile(s.ItemSpec, SourceType.Page)));
			try
			{
				engine.Extensions.AddRange(GetExtensions());
			}
			catch(FileNotFoundException ex)
			{
				Log.LogError(ex.Message);
				return false;
			}
			engine.StartInfo.RunHistory = RunHistory.LoadOrNull(historyPath);
			var result = engine.Run(true);
			if(Log.HasLoggedErrors)
				return false;
			result.History.Save(historyPath);
			var sources = result.ExecutionContext.SourceFiles.Where(s => !itemsByName.ContainsKey(s.Name)).Select(g => new
			{
				Item = CreateTaskItem(g.Name),
				SourceType = g.SourceType
			}).ToArray();
			CompileOut = sources.Where(s => s.SourceType == SourceType.Compile).Select(s => s.Item).ToArray();
			NoneOut = sources.Where(s => s.SourceType == SourceType.None).Select(s => s.Item).ToArray();
			PageOut = sources.Where(s => s.SourceType == SourceType.Page).Select(s => s.Item).ToArray();
			return true;
		}

		private TaskItem CreateTaskItem(string sourceFile)
		{
			var item = new TaskItem(sourceFile);
			item.SetMetadata("Visible", "false");
			return item;
		}
		private List<IExtension> GetExtensions()
		{
			Dictionary<string, Assembly> loadedAssemblies = new Dictionary<string, Assembly>();
			return Extensions.Select(e => e.ItemSpec.ToExtension(loadedAssemblies)).ToList();
		}
	}
}
